'' =================================================================================================
''
''   File....... neopixel_pulse_test.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 15 OCT 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                                 
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 


con

  #true, ON, OFF                                                ' digital control

  #0, LSBFIRST, MSBFIRST    


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  pulsetix                                                             
                                                                 
                                                                 
pub main                                                        
                                                                 
  setup

  ' download with [F10], open PST [F12], click [Enable], press [Enter]                                                        
                                                                 
  term.rxflush                                                  ' wait open terminal & keypress
  term.rx                                                        
  term.tx(term#CLS)

  pulsetix := US_001 * 300 / 1000 - 9                           ' test pulse 300ns

  cognew(@entry, @pulsetix)                                     ' start pulse gen cog
  time.pause(10)                                                ' let it start                                       

  repeat
    term.tx(term#HOME)
    
    term.dec(pulsetix)                                          ' display raw ticks
    term.tx(term#CLR_EOL)
    term.tx(13)

    term.dec(pulsetix * 125 / 10)                               ' display nanosecs
    term.str(string(" ns"))
    term.tx(term#CLR_EOL)
    term.tx(13)
    
    time.pause(250)                                                        
                                                                 
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 

con

dat

                        org     0

entry                   rdlong  ptix, par                       ' read adjusted pulse ticks
                        mov     dira, PIN_MASK                  ' make pin output   

                        mov     ctra, POS_DETECT                ' measure positive pulse
                        or      ctra, #15                       ' use P15
                        mov     frqa, #1
                        
again                   mov     phsa, #0                        ' clear result

pulse_out               mov     t1, ptix                        ' set pulse timing (ticks)
                        or      outa, PIN_MASK                  ' pin high
                        add     t1, cnt                         ' sync with system counter
                        waitcnt t1, #0                          ' hold
                        andn    outa, PIN_MASK                  ' pin low
                        nop
                          
                        mov     t1, phsa                        ' capture timing
                        wrlong  t1, par                         ' write to hub

                        mov     t1, QTR_SEC                     ' hold 250ms
                        add     t1, cnt
                        waitcnt t1, #0

                        jmp     #again

' -------------------------------------------------------------------------------------------------

PIN_MASK                long    1 << 15                         ' test pin P15
POS_DETECT              long    %01000 << 26                    ' time high-going pulse
QTR_SEC                 long    CLK_FREQ >> 2                   ' 250ms

ptix                    res     1
t1                      res     1

                        fit     496  

                      
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      